%define anolis_release 1

# We don't want to ship debuginfo packages for this package
# because they are firmwares of ROMS, and we should not 
# treat them as a normal program.
%global debug_package %{nil}

# QEMU ROMS with format VENDORID:DEVICEID-QEMUNAME
%global qemuroms \\\
  8086:100e-e1000 \\\
  8086:10d3-e1000e \\\
  10ec:8139-rtl8139 \\\
  1af4:1000-virtio

# We only build the ROMs if on an EFI build host. The resulting
# binary RPM will be noarch, so other archs will still be able
# to use the binary ROMs.
%global buildarches x86_64 aarch64

# Upstream don't have "releases" verion. So we're going to use the date
# as the version, and a GIT hash as the release. Generate new GIT
# snapshots using the folowing commands:
#
# $ git_hash=`git log -1 --format='%h'`
# $ date=`git log -1 --format='%cd' --date=short | tr -d -`
# $ git archive --prefix ipxe-${date}-git${git_hash}/ ${git_hash} | xz -7e > ipxe-${date}-git${git_hash}.tar.xz
#
# And then change these two:

%global git_hash de8a0821
%global date 20240119

Name:    ipxe
Version: %{date}.git%{git_hash}
Release: %{anolis_release}%{?dist}
Summary: A network boot loader

License: GPLv2 with additional permissions and BSD
URL:     http://ipxe.org/

Source0: https://github.com/%{name}/%{name}/%{name}-%{date}-git%{git_hash}.tar.xz

# Enable IPv6 for qemu's config
# Sent upstream: http://lists.ipxe.org/pipermail/ipxe-devel/2015-November/004494.html
Patch0001: 0001-build-customize-configuration.patch
Patch0002: 0002-Use-spec-compliant-timeouts.patch
Patch1001: 1001-build-without-ath9k-drivers.patch

BuildRequires: binutils-devel gcc make
BuildRequires: perl-FindBin perl-Getopt-Long perl-interpreter perl-lib
BuildRequires: edk2-tools
BuildRequires: mtools
BuildRequires: xorriso
BuildRequires: xz-devel

%ifarch x86_64
BuildRequires: syslinux
%endif

Obsoletes: gpxe <= 1.0.1

%ifarch x86_64
%package bootimgs-x86
Summary: X86 Network boot loader images in bootable USB, CD, floppy and GRUB formats
BuildArch: noarch
Provides: %{name}-bootimgs = %{version}-%{release}
Obsoletes: %{name}-bootimgs < 20200823-9.git4bd064de
Obsoletes: gpxe-bootimgs <= 1.0.1

%package roms-qemu
Summary: Network boot loader roms supported by QEMU, .rom format
BuildArch: noarch
Obsoletes: gpxe-roms-qemu <= 1.0.1

%description bootimgs-x86
iPXE is a network bootloader that is open-source and offers a direct alternative to proprietary PXE ROMs. Additionally, it comes with various additional features such as DNS, HTTP, iSCSI, and more.

This package contains the iPXE boot images in USB, CD, floppy, and PXE
UNDI formats.

%description roms-qemu
iPXE is a network bootloader that is open-source and offers a direct alternative to proprietary PXE ROMs. Additionally, it comes with various additional features such as DNS, HTTP, iSCSI, and more.

This package contains the iPXE ROMs for devices emulated by QEMU, in
.rom format.

%endif

%ifarch aarch64
%package bootimgs-aarch64
Summary: ARM Network boot loader images in bootable USB and GRUB formats
BuildArch: noarch

%description bootimgs-aarch64
iPXE is a network bootloader that is open-source and offers a direct alternative to proprietary PXE ROMs. Additionally, it comes with various additional features such as DNS, HTTP, iSCSI, and more.

This package contains the iPXE ARM boot images in USB and GRUB formats.

%endif

%description
iPXE is a network bootloader that is open-source and offers a direct alternative to proprietary PXE ROMs. Additionally, it comes with various additional features such as DNS, HTTP, iSCSI, and more.

%prep
%autosetup -n %{name}-%{date}-git%{git_hash} -p1

%build
cd src
ipxe_build() {
    %make_build \
        GITVERSION=%{git_hash} \
        NO_WERROR=1 V=1 \
        "$@"
}

%ifarch x86_64

ipxe_build bin-x86_64-efi/snponly.efi

ipxe_build ISOLINUX_BIN=/usr/share/syslinux/isolinux.bin \
        bin/undionly.kpxe

# build roms with efi support for qemu
mkdir bin-combined
for rom in %{qemuroms}; do
  pci_id=$(echo "$rom" | cut -d "-" -f 1)
  vendor_id=${pci_id%:*}
  device_id=${pci_id#*:}

  ipxe_build CONFIG=qemu bin/${vendor_id}${device_id}.rom
  ipxe_build CONFIG=qemu bin-x86_64-efi/${vendor_id}${device_id}.efidrv

  EfiRom -f "0x$vendor_id" -i "0x$device_id" --pci23 \
         -ec bin-x86_64-efi/${vendor_id}${device_id}.efidrv \
         -o  bin-combined/${vendor_id}${device_id}.eficrom
  util/catrom.pl \
      bin/${vendor_id}${device_id}.rom \
      bin-combined/${vendor_id}${device_id}.eficrom \
      > bin-combined/${vendor_id}${device_id}.rom
  EfiRom -d bin-combined/${vendor_id}${device_id}.rom

  # truncate to 256KiB and verify
  truncate -s \>256K bin-combined/${vendor_id}${device_id}.rom
  test $(stat -c '%s' bin-combined/${vendor_id}${device_id}.rom) -le $((256 * 1024))
done

%endif

%ifarch aarch64
ipxe_build bin-arm64-efi/snponly.efi
%endif

%install
%ifarch x86_64
mkdir -p %{buildroot}/%{_datadir}/%{name}/
mkdir -p %{buildroot}/%{_datadir}/%{name}.efi/
pushd src/bin/

install -m 0644 -p undionly.kpxe %{buildroot}/%{_datadir}/%{name}/

popd

install -m 0644 -p src/bin-x86_64-efi/snponly.efi %{buildroot}/%{_datadir}/%{name}/ipxe-snponly-x86_64.efi

mkdir -p %{buildroot}%{_datadir}/%{name}/qemu/

for rom in %{qemuroms}; do
  # the roms supported by qemu will be packaged separatedly
  # remove from the main rom list and add them to qemu.list
  pci_id=$(echo "$rom" | cut -d "-" -f 1)
  qemuname=$(echo "$rom" | cut -d "-" -f 2)
  vendor_id=${pci_id%:*}
  device_id=${pci_id#*:}

  echo %{_datadir}/%{name}/${vendor_id}${device_id}.rom >> qemu.rom.list

  install -m 0644 -p src/bin/${vendor_id}${device_id}.rom %{buildroot}/%{_datadir}/%{name}/
  install -m 0644 -p src/bin-combined/${vendor_id}${device_id}.rom %{buildroot}/%{_datadir}/%{name}.efi/
  echo %{_datadir}/%{name}.efi/${vendor_id}${device_id}.rom >> qemu.rom.list

  # Set up symlinks with expected qemu firmware names
  ln -s ../../ipxe/${vendor_id}${device_id}.rom %{buildroot}%{_datadir}/%{name}/qemu/pxe-${qemuname}.rom
  ln -s ../../ipxe.efi/${vendor_id}${device_id}.rom %{buildroot}%{_datadir}/%{name}/qemu/efi-${qemuname}.rom
done

%endif

%ifarch aarch64
mkdir -p %{buildroot}/%{_datadir}/%{name}/arm64-efi
install -m 0644 -p src/bin-arm64-efi/snponly.efi %{buildroot}/%{_datadir}/%{name}/arm64-efi/snponly.efi

%endif

%ifarch x86_64
%files bootimgs-x86
%license COPYING COPYING.GPLv2 COPYING.UBDL
%dir %{_datadir}/%{name}
%{_datadir}/%{name}/undionly.kpxe
%{_datadir}/%{name}/ipxe-snponly-x86_64.efi

%files roms-qemu -f qemu.rom.list
%license COPYING COPYING.GPLv2 COPYING.UBDL
%dir %{_datadir}/%{name}
%dir %{_datadir}/%{name}.efi
%{_datadir}/%{name}/qemu
%endif

%ifarch aarch64
%files bootimgs-aarch64
%dir %{_datadir}/%{name}
%dir %{_datadir}/%{name}/arm64-efi
%{_datadir}/%{name}/arm64-efi/snponly.efi
%endif

%changelog
* Wed Feb 19 2025 Xiaoping Liu <lxp01404969@linux.alibaba.com> - 20240119-1.gitde8a0821
- Sync up from CentOS Stream ipxe-20240119-5.gitde8a0821 (tdawson@redhat.com)

* Sat Apr 8 2023 Kun(llfl) <llfl@linux.alibaba.com> -  20220210.git64113751-3
- optimize spec file

* Tue Mar 07 2023 happy_orange <songnannan@linux.alibaba.com> - 20220210.git64113751-2
- change the path of license files

* Fri Jul 15 2022 happy_orange <songnannan@linux.alibaba.com> - 20220210.git64113751-1
- Init pacakge from upstream
